             .title        "JF-16 (INES 78) Reader Plugin"


;11/03/09
;Written by BootGod
;Fixed to 128K PRG / 128K CHR. Plugin should work for Irem's Holy Diver as well.
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:	.equ 00e0h
address_lo:	.equ 00e0h
address_hi:	.equ 00e1h
curbank:	.equ 00e2h
banksleft:	.equ 00e3h
numpages_lo:	.equ 00e4h
numpages_hi:	.equ 00e5h
reg:		.equ 00e6h

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Jaleco JF-16 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,0 >> 4
             .db 128 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             lda #4
             jsr send_byte	;mirroring is mapper ctrl

;PRG ROM
             

got_prg:     lda #0		;send PRG header
             jsr send_byte
             lda #2		;fixed to 128K
             jsr send_byte
             lda #1
             jsr send_byte
             
             lda #0
             sta curbank

             lda #8		;8 x 16K
             sta banksleft
             
next_prg:    lda curbank
             jsr wr_bank

             lda #0		;set up read from $8000
             sta address_lo
             lda #80h
             sta address_hi

             ldy #0		;set counters for 16K dump
             ldx #64

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             inc curbank
             dec banksleft
             bne next_prg
                          
;CHR ROM

             
got_chr:     lda #0		;send CHR header
             jsr send_byte
             lda #2		;fixed to 128K
             jsr send_byte
             lda #002h
             jsr send_byte

             lda #0
             sta curbank

             lda #16		;16 x 8K
             sta banksleft
             
next_chr:    lda curbank
             asl a		;shift 4 bits, CHR reg is upper nibble
             asl a
             asl a
             asl a
             jsr wr_bank
             inc curbank
             lda #0
             jsr wr_ppu
             lda 2007h

             ldx #32		;set counters for 8K dump
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             dec banksleft
             bne next_chr
              
done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts


;sub for writing bank (compensates for bus conflicts)

wr_bank:     sta reg		;store value to be written to reg

             lda #0
             sta address_lo
             lda #0C0h
             sta address_hi	;set address to $C000

             ldx #64		;setup to search last 16K bank
             ldy #0

wrb_loop:    lda (address),y
             and #0f7h		;mask it to just the bits we need to match
             cmp reg		;check if current byte matches what we want to write
             beq found_reg
             iny
             bne wrb_loop
             inc address_hi
             bne wrb_loop

no_reg:      lda reg		;didn't find place to write, try it anyways
             sta 0c000h
             jmp wrb_return

found_reg:   lda (address),y	;found value equal to reg, write it.
             sta (address),y

wrb_return:  rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
